#ifndef _NETFILTER_CTHELPER_H
#define _NETFILTER_CTHELPER_H

#include <libnetfilter_cthelper/linux_nfnetlink_cthelper.h>

struct nfcth_handle;
struct nfcth_expect;

struct nfcth_helper {
	/* library internal data, not to be touched by user */
	struct list_head list;
	struct nfcth_handle *h;
	struct nfq_q_handle *qh;
	u_int16_t queue_id;
	/* end of library internal data */

	/* public data */
	const char *name;		/* name of the helper */
	unsigned int max_expected;	/* max num of expected conns */
	unsigned int timeout;		/* timeout for expecteds */

	/* Mask things which we will help (compared against server response) */
	struct nfct_tuple tuple;
	struct nfct_tuple mask;

	/* Function to call when data passes; return verdict, or -1
	 * to invalidate. */
	int (*help)(struct nfq_data *nfad);
};

extern struct nfcth_handle *nfcth_open_nfnl(struct nfnl_handle *nfnlh);
extern struct nfcth_handle *nfcth_open(void);
extern void nfcth_close(struct nfcth_handle *hh);

extern struct nfnl_handle *nfcth_nfnlh(struct nfcth_handle *h);
extern int nfcth_fd(struct nfcth_handle *h);

extern int nfcth_handle_packet(struct nfcth_handle *h, char *buf, int len);

extern int nfcth_helper_register(struct nfcth_handle *h,
				 struct nfcth_helper *helper);
extern void nfcth_helper_unregister(struct nfcth_helper *helper);

extern int nfcth_expect_related(struct nfcth_helper *helper,
				struct nfcth_expect *exp);
extern void nfcth_unexpect_related(struct nfcth_expect *exp);

#endif /* _NETFILTER_CTHELPER_H */
